/*
 * This file is a part of the CaosDB Project.
 *
 * Copyright (C) 2023 IndiScale GmbH <info@indiscale.com>
 * Copyright (C) 2023 Timm Fitschen <t.fitschen@indiscale.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program. If not, see <https://www.gnu.org/licenses/>.
 */

DROP PROCEDURE IF EXISTS setFileProperties;
DELIMITER //
/**
 * Insert/Update file properties.
 *
 * If ENTITY_VERSIONING is enabled the old file properties are moved to
 * `archive_files`.
 *
 * Parameters
 * ----------
 * FileID
 *   The entity's id.
 * FileHash
 *   The SHA512 Hash of the file (or NULL for directories).
 * FileCheckedTimestamp
 *   The timestamp when the hash was last checked.
 * FileSize
 *   The byte size (or NULL for directories).
 * FilePath
 *   The path of the object.
 * FileMimeType
 *   The mimetype (use "inode/directory" for directories, use NULL for unknown)
 * FileStorageID
 *   The ID of the back-end file storage where this object is located.
 * FileKey
 *   The file storage key of this object.
 * FileParentID
 *   The entity id of the parent directory (or NULL).
 */
CREATE PROCEDURE setFileProperties (
    in FileID INT UNSIGNED,
    in FilePath VARCHAR(5000),
    in FileSize BIGINT UNSIGNED,
    in FileHash VARCHAR(255),
    in FileChecked BIGINT,
    in FileMimeType VARBINARY(255),
    in FileStorageId VARBINARY(255),
    in FileKey VARBINARY(40000),
    in FileParentID INT UNSIGNED,
    in FileHashAlgo VARCHAR(255)
)
BEGIN
    DECLARE IVersion INT UNSIGNED DEFAULT NULL;
    DECLARE InternalEntityID INT UNSIGNED DEFAULT NULL;

    SELECT internal_id INTO InternalEntityID FROM entity_ids WHERE id = FileID;
    IF is_feature_config("ENTITY_VERSIONING", "ENABLED") THEN
        SELECT max(e._iversion) INTO IVersion
            FROM entity_version AS e
            WHERE e.entity_id = InternalEntityID;

        INSERT INTO archive_files (file_id, path, size, hash,
                checked_timestamp, mimetype, file_storage_id, file_key,
                _iversion, hash_algorithm)
            SELECT file_id, path, size, hash, checked_timestamp, mimetype,
                file_storage_id, file_key, IVersion AS _iversion, hash_algorithm
            FROM files
            WHERE file_id = InternalEntityID;
    END IF;

    DELETE FROM files WHERE file_id = InternalEntityID;

    IF FilePath IS NOT NULL THEN
        INSERT INTO files (file_id, path, size, hash, checked_timestamp,
                mimetype, file_storage_id, file_key, hash_algorithm)
            VALUES (InternalEntityID, FilePath, FileSize, unhex(FileHash), FileChecked,
                FileMimeType, FileStorageId, FileKey, FileHashAlgo);
    END IF;

END //
DELIMITER ;
